@@ -15,8 +15,8 @@ DiggleGratton <- local({
     stopifnot(is.numeric(rho))
     stopifnot(delta < rho)
     # sort in increasing order of x coordinate
-    oX <- order(X$x)
-    oY <- order(Y$x)
+    oX <- fave.order(X$x)
+    oY <- fave.order(Y$x)
     Xsort <- X[oX]
     Ysort <- Y[oY]
     idXsort <- idX[oX]
