@@ -24,24 +24,23 @@ edges2triangles <- function(iedge, jedge, nvert=max(iedge, jedge),
   # zero length data
   if(length(iedge) == 0) return(matrix(, nrow=0, ncol=3))
   # sort in increasing order of 'iedge'
-  oi <- order(iedge)
+  oi <- fave.order(iedge)
   iedge <- iedge[oi]
   jedge <- jedge[oi]
-  # convert to C indexing
-  ii <- iedge - 1
-  jj <- jedge - 1
   # call C
-  storage.mode(nvert) <- storage.mode(ii) <- storage.mode(jj) <- "integer"
+  storage.mode(nvert) <- storage.mode(iedge) <- storage.mode(jedge) <- "integer"
   if(!usefriends) {
-    zz <- .Call("triograph", nv=nvert, iedge=ii, jedge=jj, PACKAGE="spatstat")
+    zz <- .Call("triograph",
+                nv=nvert, iedge=iedge, jedge=jedge,
+                PACKAGE="spatstat")
   } else {
     fr <- as.logical(friendly)
     storage.mode(fr) <- "integer"
-    zz <- .Call("trioxgraph", nv=nvert, iedge=ii, jedge=jj, friendly=fr,
+    zz <- .Call("trioxgraph",
+                nv=nvert, iedge=iedge, jedge=jedge, friendly=fr,
                 PACKAGE="spatstat")
   }
-  # convert back to R indexing
-  mat <- as.matrix(as.data.frame(zz)) + 1
+  mat <- as.matrix(as.data.frame(zz))
   return(mat)
 }
 
@@ -65,18 +64,14 @@ trianglediameters <- function(iedge, jedge, edgelength, ...,
                       j=integer(0),
                       k=integer(0),
                       diam=numeric(0)))
-  # convert to C indexing
-  ii <- iedge - 1
-  jj <- jedge - 1
-  eij <- edgelength
+
   # call C
-  storage.mode(nvert) <- storage.mode(ii) <- storage.mode(jj) <- "integer"
-  storage.mode(eij) <- "double"
-  zz <- .Call("triDgraph", nv=nvert, iedge=ii, jedge=jj, edgelength=eij,
+  storage.mode(nvert) <- storage.mode(iedge) <- storage.mode(jedge) <- "integer"
+  storage.mode(edgelength) <- "double"
+  zz <- .Call("triDgraph",
+              nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength,
               PACKAGE="spatstat")
   df <- as.data.frame(zz)
   colnames(df) <- c("i", "j", "k", "diam")
-  # convert back to R indexing
-  df[, 1:3] <- df[, 1:3] + 1
   return(df)
 }
