@@ -102,8 +102,8 @@ strausscounts <- function(U, X, r, EqualPairs=NULL) {
 crosspaircounts <- function(X, Y, r) {
   stopifnot(is.numeric(r))
   # sort in increasing order of x coordinate
-  oX <- order(X$x)
-  oY <- order(Y$x)
+  oX <- fave.order(X$x)
+  oY <- fave.order(Y$x)
   Xsort <- X[oX]
   Ysort <- Y[oY]
   nX <- npoints(X)
