@@ -4,12 +4,14 @@
 
   Nearest Neighbour Distances in m dimensions
 
-  $Revision: 1.5 $     $Date: 2011/11/20 04:09:19 $
+  $Revision: 1.7 $     $Date: 2012/04/06 09:45:37 $
 
   Argument x is an m * n matrix 
   with columns corresponding to points
   and rows corresponding to coordinates.
 
+  Spatial dimension m must be > 1
+
   THE FOLLOWING FUNCTIONS ASSUME THAT THE ROWS OF x 
   ARE SORTED IN ASCENDING ORDER OF THE FIRST COLUMN
 
@@ -27,6 +29,7 @@
 #include <R.h>
 #include <R_ext/Utils.h>
 #include <math.h>
+#include "chunkloop.h"
 
 #ifndef FALSE
 #define TRUE 1
@@ -36,110 +39,104 @@
 double sqrt();
 
 void nndMD(n, m, x, nnd, huge)
-     /* inputs */
+/* inputs */
      int *n, *m;
      double *x, *huge;
      /* output */
      double *nnd;
 { 
-  int npoints, mdimen, i, j, left, right, leftpos, rightpos;
-  double dmin, d2, d2min, hu, hu2, xi0, dx0, dxj;
+  int npoints, mdimen, i, j, left, right, leftpos, rightpos, maxchunk;
+  double d2, d2min, hu, hu2, xi0, dx0, dxj;
   double *xi;
 
   npoints = *n;
-  mdimen  = *m;
+  mdimen  = *m; 
   xi = (double *) R_alloc((size_t) mdimen, sizeof(double));
   /*  dx = (double *) R_alloc((size_t) mdimen, sizeof(double)); */
 
   hu = *huge;
   hu2 = hu * hu;
 
-  for(i = 0; i < npoints; i++) {
-
+  OUTERCHUNKLOOP(i, npoints, maxchunk, 16384) {
     R_CheckUserInterrupt();
+    INNERCHUNKLOOP(i, npoints, maxchunk, 16384) {
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\ni=%d\n", i); 
+      Rprintf("\ni=%d\n", i); 
 #endif
 
-    dmin = hu;
-    d2min = hu2;
+      d2min = hu2;
 
-    for(j = 0; j < mdimen; j++)
-      xi[j] = x[i * mdimen + j];
-    xi0 = xi[0];
+      for(j = 0; j < mdimen; j++)
+	xi[j] = x[i * mdimen + j];
+      xi0 = xi[0];
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n (");
-    for(j = 0; j < mdimen; j++)
-      Rprintf("%lf, ", x[i * mdimen + j]);
-    Rprintf(")\n");
+      Rprintf("\n (");
+      for(j = 0; j < mdimen; j++)
+	Rprintf("%lf, ", x[i * mdimen + j]);
+      Rprintf(")\n");
 #endif
 
     
-    /* search backward */
-    if(i > 0) {
-      for(left = i - 1;
-	  left >= 0 && (dx0 = (xi0 - x[left * mdimen])) < dmin ;
-	  --left)
-	{
+      /* search backward */
+      if(i > 0) {
+	for(left = i - 1; left >= 0; --left) {
 
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("L=%d, dmin=%lf\n", left, dmin);
+	  Rprintf("L=%d, d2min=%lf\n", left, d2min);
 #endif
-
+	  dx0 = xi0 - x[left * mdimen];
 	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    leftpos = left * mdimen;
-	    for(j = 1; j < mdimen && d2 < d2min; j++) {
-	      dxj = xi[j] - x[leftpos + j];
-	      d2 += dxj * dxj;
-	    }
+	  if(d2 > d2min)
+	    break;
+
+	  leftpos = left * mdimen;
+	  for(j = 1; j < mdimen && d2 < d2min; j++) {
+	    dxj = xi[j] - x[leftpos + j];
+	    d2 += dxj * dxj;
 	  }
 
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tupdating dmin=%lf\n", dmin);
+	    Rprintf("\tupdating d2min=%lf\n", d2min);
 #endif
 	  }
 	}
-    }
-
-    /* search forward */
-    if(i < npoints - 1) {
-      for(right = i + 1;
-	  right < npoints && (dx0 = (x[right * mdimen] - xi0)) < dmin ;
-	  ++right)
-	{
+      }
 
+      /* search forward */
+      if(i < npoints - 1) {
+	for(right = i + 1; right < npoints; ++right) {
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("R=%d, dmin=%lf\n", right, dmin);
+	  Rprintf("R=%d, d2min=%lf\n", right, d2min);
 #endif
-	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    rightpos = right * mdimen;
-	    for(j = 1; j < mdimen && d2 < d2min; j++) {
-	      dxj = xi[j] - x[rightpos + j];
-	      d2 += dxj * dxj;
-	    }
+	  dx0 = x[right * mdimen] - xi0;
+	  d2  = dx0 * dx0;
+	  if(d2 > d2min)
+	    break;
+
+	  rightpos = right * mdimen;
+	  for(j = 1; j < mdimen && d2 < d2min; j++) {
+	    dxj = xi[j] - x[rightpos + j];
+	    d2 += dxj * dxj;
 	  }
 
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tupdating dmin=%lf\n", dmin);
+	    Rprintf("\tupdating d2min=%lf\n", d2min);
 #endif
 	  }
 	}
-    }
+      }
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n");
+      Rprintf("\n");
 #endif
 
-    nnd[i] = dmin;
+      nnd[i] = sqrt(d2min);
+    }
   }
 }
 
@@ -148,15 +145,15 @@ void nndMD(n, m, x, nnd, huge)
 */
 
 void nnwMD(n, m, x, nnd, nnwhich, huge)
-     /* inputs */
+/* inputs */
      int *n, *m;
      double *x, *huge;
      /* output */
      double *nnd;
      int *nnwhich;
 { 
-  int npoints, mdimen, i, j, left, right, leftpos, rightpos, which;
-  double dmin, d2, d2min, hu, hu2, xi0, dx0, dxj;
+  int npoints, mdimen, i, j, left, right, leftpos, rightpos, which, maxchunk;
+  double d2, d2min, hu, hu2, xi0, dx0, dxj;
   double *xi;
 
   npoints = *n;
@@ -167,96 +164,88 @@ void nnwMD(n, m, x, nnd, nnwhich, huge)
   hu = *huge;
   hu2 = hu * hu;
 
-  for(i = 0; i < npoints; i++) {
-
+  OUTERCHUNKLOOP(i, npoints, maxchunk, 16384) {
     R_CheckUserInterrupt();
+    INNERCHUNKLOOP(i, npoints, maxchunk, 16384) {
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\ni=%d\n", i); 
+      Rprintf("\ni=%d\n", i); 
 #endif
 
-    dmin = hu;
-    d2min = hu2;
-    which = -1;
+      d2min = hu2;
+      which = -1;
 
-    for(j = 0; j < mdimen; j++)
-      xi[j] = x[i * mdimen + j];
-    xi0 = xi[0];
-
-    /* search backward */
-    if(i > 0) {
-      for(left = i - 1;
-	  left >= 0 && (dx0 = (xi0 - x[left * mdimen])) < dmin ;
-	  --left)
-	{
+      for(j = 0; j < mdimen; j++)
+	xi[j] = x[i * mdimen + j];
+      xi0 = xi[0];
 
+      /* search backward */
+      if(i > 0) {
+	for(left = i - 1; left >= 0; --left) {
 #ifdef SPATSTAT_DEBUG
 	  Rprintf("L");
 #endif
-
+	  dx0 = xi0 - x[left * mdimen];
 	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    leftpos = left * mdimen;
-	    for(j = 1; j < mdimen && d2 < d2min; j++) {
-	      dxj = xi[j] - x[leftpos + j];
-	      d2 += dxj * dxj;
-	    }
+	  if(d2 > d2min)
+	    break;
+	  leftpos = left * mdimen;
+	  for(j = 1; j < mdimen && d2 < d2min; j++) {
+	    dxj = xi[j] - x[leftpos + j];
+	    d2 += dxj * dxj;
 	  }
-
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 	    which = left;
 	  }
 	}
-    }
+      }
 
-    /* search forward */
-    if(i < npoints - 1) {
-      for(right = i + 1;
-	  right < npoints && (dx0 = (x[right * mdimen] - xi0)) < dmin ;
-	  ++right)
-	{
+      /* search forward */
+      if(i < npoints - 1) {
+	for(right = i + 1; right < npoints; ++right) {
 
 #ifdef SPATSTAT_DEBUG
 	  Rprintf("R");
 #endif
+	  dx0 = x[right * mdimen] - xi0;
 	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    rightpos = right * mdimen;
-	    for(j = 1; j < mdimen && d2 < d2min; j++) {
-	      dxj = xi[j] - x[rightpos + j];
-	      d2 += dxj * dxj;
-	    }
-	  }
+	  if(d2 > d2min)
+	    break;
 
+	  rightpos = right * mdimen;
+	  for(j = 1; j < mdimen && d2 < d2min; j++) {
+	    dxj = xi[j] - x[rightpos + j];
+	    d2 += dxj * dxj;
+	  }
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 	    which = right;
 	  }
 	}
-    }
+      }
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n");
+      Rprintf("\n");
 #endif
 
-    nnd[i] = dmin;
-    nnwhich[i] = which;
+      nnd[i] = sqrt(d2min);
+      /* convert index to R convention */
+      nnwhich[i] = which + 1;
+    }
   }
 }
 
 /* 
    nnXwMD:  for TWO point patterns X and Y,
-              find the nearest neighbour 
-	      (from each point of X to the nearest point of Y)
-	      returning both the distance and the identifier
+   find the nearest neighbour 
+   (from each point of X to the nearest point of Y)
+   returning both the distance and the identifier
 
    Requires both patterns to be sorted in order of increasing z coord
 */
 
 void nnXwMD(m, n1, x1, n2, x2, nnd, nnwhich, huge)
-     /* inputs */
+/* inputs */
      int *m, *n1, *n2;
      double *x1, *x2, *huge;
      /* outputs */
@@ -264,8 +253,9 @@ void nnXwMD(m, n1, x1, n2, x2, nnd, nnwhich, huge)
      int *nnwhich;
 { 
   int mdimen, npoints1, npoints2, i, ell, jleft, jright, jwhich, lastjwhich;
-  double dmin, d2, d2min, x1i0, dx0, dxell, hu, hu2;
+  double d2, d2min, x1i0, dx0, dxell, hu, hu2;
   double *x1i;
+  int maxchunk;
 
   hu = *huge;
   hu2 = hu * hu;
@@ -282,66 +272,62 @@ void nnXwMD(m, n1, x1, n2, x2, nnd, nnwhich, huge)
 
   lastjwhich = 0;
 
-  for(i = 0; i < npoints1; i++) {
+  OUTERCHUNKLOOP(i, npoints1, maxchunk, 16384) {
     R_CheckUserInterrupt();
-    dmin = hu;
-    d2min = hu2;
-    jwhich = -1;
-    for(ell = 0; ell < mdimen; ell++) 
-      x1i[ell] = x1[i * mdimen + ell];
-    x1i0 = x1i[0];
+    INNERCHUNKLOOP(i, npoints1, maxchunk, 16384) {
+      d2min = hu2;
+      jwhich = -1;
+      for(ell = 0; ell < mdimen; ell++) 
+	x1i[ell] = x1[i * mdimen + ell];
+      x1i0 = x1i[0];
 
-    /* search backward from previous nearest neighbour */
-    if(lastjwhich > 0) {
-      for(jleft = lastjwhich - 1;
-	  jleft >= 0 && (dx0 = (x1i0 - x2[jleft * mdimen])) < dmin ;
-	  --jleft)
-	{
+      /* search backward from previous nearest neighbour */
+      if(lastjwhich > 0) {
+	for(jleft = lastjwhich - 1; jleft >= 0; --jleft) {
+	  dx0 = x1i0 - x2[jleft * mdimen];
 	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
-	      dxell = x1i[ell] - x2[jleft * mdimen + ell];
-	      d2 += dxell * dxell;
-	    }
+	  if(d2 > d2min)
+	    break;
+	  for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
+	    dxell = x1i[ell] - x2[jleft * mdimen + ell];
+	    d2 += dxell * dxell;
 	  }
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 	    jwhich = jleft;
 	  }
 	}
-    }
+      }
 
-    /* search forward from previous nearest neighbour  */
-    if(lastjwhich < npoints2) {
-      for(jright = lastjwhich;
-	  jright < npoints2 && (dx0 = (x2[jright * mdimen] - x1i0)) < dmin ;
-	  ++jright)
-	{
+      /* search forward from previous nearest neighbour  */
+      if(lastjwhich < npoints2) {
+	for(jright = lastjwhich; jright < npoints2; ++jright) {
+	  dx0 = x2[jright * mdimen] - x1i0;
 	  d2 = dx0 * dx0;
-	  if(mdimen > 1) {
-	    for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
-	      dxell = x1i[ell] - x2[jright * mdimen + ell];
-	      d2 += dxell * dxell;
-	    }
+	  if(d2 > d2min) 
+	    break;
+	  for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
+	    dxell = x1i[ell] - x2[jright * mdimen + ell];
+	    d2 += dxell * dxell;
 	  }
 	  if (d2 < d2min) {
 	    d2min = d2;
-	    dmin = sqrt(d2);
 	    jwhich = jright;
 	  }
 	}
+      }
+      nnd[i] = sqrt(d2min);
+      nnwhich[i] = jwhich;
+      lastjwhich = jwhich;
     }
-    nnd[i] = dmin;
-    nnwhich[i] = jwhich;
-    lastjwhich = jwhich;
   }
 }
 
+
 /* 
    nnXxMD:  similar to nnXwMD
-              but allows X and Y to include common points
-	      (which are not to be counted as neighbours)
+   but allows X and Y to include common points
+   (which are not to be counted as neighbours)
 
    Code numbers id1, id2 are attached to the patterns X and Y respectively, 
    such that
@@ -351,7 +337,7 @@ void nnXwMD(m, n1, x1, n2, x2, nnd, nnwhich, huge)
 */
 
 void nnXxMD(m, n1, x1, id1, n2, x2, id2, nnd, nnwhich, huge)
-     /* inputs */
+/* inputs */
      int *m, *n1, *n2;
      double *x1, *x2, *huge;
      int *id1, *id2;
@@ -360,8 +346,9 @@ void nnXxMD(m, n1, x1, id1, n2, x2, id2, nnd, nnwhich, huge)
      int *nnwhich;
 { 
   int mdimen, npoints1, npoints2, i, ell, jleft, jright, jwhich, lastjwhich, id1i;
-  double dmin, d2, d2min, x1i0, dx0, dxell, hu, hu2;
+  double d2, d2min, x1i0, dx0, dxell, hu, hu2;
   double *x1i;
+  int maxchunk;
 
   hu = *huge;
   hu2 = hu * hu;
@@ -378,71 +365,65 @@ void nnXxMD(m, n1, x1, id1, n2, x2, id2, nnd, nnwhich, huge)
 
   lastjwhich = 0;
 
-  for(i = 0; i < npoints1; i++) {
-
+  OUTERCHUNKLOOP(i, npoints1, maxchunk, 16384) {
     R_CheckUserInterrupt();
+    INNERCHUNKLOOP(i, npoints1, maxchunk, 16384) {
+      d2min = hu2;
+      jwhich = -1;
+      id1i   = id1[i];
+      for(ell = 0; ell < mdimen; ell++) 
+	x1i[ell] = x1[i * mdimen + ell];
+      x1i0 = x1i[0];
 
-    dmin = hu;
-    d2min = hu2;
-    jwhich = -1;
-    id1i   = id1[i];
-    for(ell = 0; ell < mdimen; ell++) 
-      x1i[ell] = x1[i * mdimen + ell];
-    x1i0 = x1i[0];
-
-    /* search backward from previous nearest neighbour */
-    if(lastjwhich > 0) {
-      for(jleft = lastjwhich - 1;
-	  jleft >= 0 && (dx0 = (x1i0 - x2[jleft * mdimen])) < dmin ;
-	  --jleft)
-	{
+      /* search backward from previous nearest neighbour */
+      if(lastjwhich > 0) {
+	for(jleft = lastjwhich - 1; jleft >= 0; --jleft) {
+	  dx0 = x1i0 - x2[jleft * mdimen];
+	  d2 = dx0 * dx0;
+	  if(d2 > d2min)
+	    break;
 	  /* do not compare identical points */
 	  if(id2[jleft] != id1i) {
-	    d2 = dx0 * dx0;
-	    if(mdimen > 1) {
-	      for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
-		dxell = x1i[ell] - x2[jleft * mdimen + ell];
-		d2 += dxell * dxell;
-	      }
+	    for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
+	      dxell = x1i[ell] - x2[jleft * mdimen + ell];
+	      d2 += dxell * dxell;
 	    }
 	    if (d2 < d2min) {
 	      d2min = d2;
-	      dmin = sqrt(d2);
 	      jwhich = jleft;
 	    }
 	  }
 	}
-    }
+      }
 
-    /* search forward from previous nearest neighbour  */
-    if(lastjwhich < npoints2) {
-      for(jright = lastjwhich;
-	  jright < npoints2 && (dx0 = (x2[jright * mdimen] - x1i0)) < dmin ;
-	  ++jright)
-	{
+      /* search forward from previous nearest neighbour  */
+      if(lastjwhich < npoints2) {
+	for(jright = lastjwhich; jright < npoints2; ++jright) {
+	  dx0 = x2[jright * mdimen] - x1i0;
+	  d2 = dx0 * dx0;
+	  if(d2 > d2min) 
+	    break;
 	  /* do not compare identical points */
 	  if(id2[jright] != id1i) {	  
-	    d2 = dx0 * dx0;
-	    if(mdimen > 1) {
-	      for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
-		dxell = x1i[ell] - x2[jright * mdimen + ell];
-		d2 += dxell * dxell;
-	      }
+	    for(ell = 1; ell < mdimen && d2 < d2min; ell++) {
+	      dxell = x1i[ell] - x2[jright * mdimen + ell];
+	      d2 += dxell * dxell;
 	    }
 	    if (d2 < d2min) {
 	      d2min = d2;
-	      dmin = sqrt(d2);
 	      jwhich = jright;
 	    }
 	  }
 	}
+      }
+      nnd[i] = sqrt(d2min);
+      nnwhich[i] = jwhich;
+      lastjwhich = jwhich;
     }
-    nnd[i] = dmin;
-    nnwhich[i] = jwhich;
-    lastjwhich = jwhich;
   }
 }
 
+
 /* 
    knndMD
 
@@ -451,15 +432,15 @@ void nnXxMD(m, n1, x1, id1, n2, x2, id2, nnd, nnwhich, huge)
 */
 
 void knndMD(n, m, kmax, x, nnd, huge)
-     /* inputs */
+/* inputs */
      int *n, *m, *kmax;
      double *x, *huge;
      /* output matrix (kmax * npoints) */
      double *nnd;
 { 
-  int npoints, mdimen, nk, nk1, i, j, k, k1, left, right, unsorted;
-  double d2, dminK, d2minK, xi0, dx0, dxj, hu, hu2, tmp, tmp2;
-  double *dmin, *d2min, *xi;
+  int npoints, mdimen, nk, nk1, i, j, k, k1, left, right, unsorted, maxchunk;
+  double d2, d2minK, xi0, dx0, dxj, hu, hu2, tmp;
+  double *d2min, *xi;
 
   hu = *huge;
   hu2 = hu * hu;
@@ -470,191 +451,168 @@ void knndMD(n, m, kmax, x, nnd, huge)
   nk1     = nk - 1;
 
   /* 
-     create space to store the nearest neighbour distances
+     create space to store the squared k-th nearest neighbour distances
      for the current point
   */
 
-  dmin = (double *) R_alloc((size_t) nk, sizeof(double));
   d2min = (double *) R_alloc((size_t) nk, sizeof(double));
 
   /* 
      scratch space
   */
   xi = (double *) R_alloc((size_t) mdimen, sizeof(double));
-  /*  dx = (double *) R_alloc((size_t) mdimen, sizeof(double)); */
 
   /* loop over points */
 
-  for(i = 0; i < npoints; i++) {
-
+  OUTERCHUNKLOOP(i, npoints, maxchunk, 16384) {
     R_CheckUserInterrupt();
-
+    INNERCHUNKLOOP(i, npoints, maxchunk, 16384) {
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\ni=%d\n", i); 
+      Rprintf("\ni=%d\n", i); 
 #endif
 
-    /* initialise nn distances */
+      /* initialise nn distances */
 
-    dminK  = hu;
-    d2minK = hu2;
-    for(k = 0; k < nk; k++) {
-      dmin[k] = hu;
-      d2min[k] = hu2;
-    }
+      d2minK = hu2;
+      for(k = 0; k < nk; k++)
+	d2min[k] = hu2;
 
-    for(j = 0; j < mdimen; j++)
-      xi[j] = x[i* mdimen + j];
-    xi0 = xi[0];
+      for(j = 0; j < mdimen; j++)
+	xi[j] = x[i* mdimen + j];
+      xi0 = xi[0];
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n (");
-    for(j = 0; j < mdimen; j++)
-      Rprintf("%lf, ", xi[j]);
-    Rprintf(")\n");
+      Rprintf("\n (");
+      for(j = 0; j < mdimen; j++)
+	Rprintf("%lf, ", xi[j]);
+      Rprintf(")\n");
 #endif
 
-    /* search backward */
-    for(left = i - 1;
-        left >= 0 && (dx0 = (xi0 - x[left * mdimen])) < dminK ;
-	--left)
-      {
-
+      /* search backward */
+      for(left = i - 1; left >= 0; --left) {
+	dx0 = xi0 - x[left * mdimen];
 	d2 = dx0 * dx0; 
+	if(d2 > d2minK)
+	  break;
 #ifdef SPATSTAT_DEBUG
 	Rprintf("L=%d\n", left);
 	Rprintf("\t 0 ");
 #endif
-	if(mdimen > 1) {
-	  for(j = 1; j < mdimen && d2 < d2minK; j++) {
+	for(j = 1; j < mdimen && d2 < d2minK; j++) {
 #ifdef SPATSTAT_DEBUG
-	    Rprintf("%d ", j);
+	  Rprintf("%d ", j);
 #endif
-	    dxj = xi[j] - x[left * mdimen + j];
-	    d2 += dxj * dxj;
-	  }
+	  dxj = xi[j] - x[left * mdimen + j];
+	  d2 += dxj * dxj;
 	}
 #ifdef SPATSTAT_DEBUG
-	Rprintf("\n\t sqrt(d2)=%lf\n", sqrt(d2));
+	Rprintf("\n\t d2=%lf\n", d2);
 #endif
 	if (d2 < d2minK) {
 	  /* overwrite last entry */
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tsqrt(d2)=%lf overwrites dmin[%d] = %lf\n", 
-		  sqrt(d2), nk1, dmin[nk1]);
+	  Rprintf("\td2=%lf overwrites d2min[%d] = %lf\n", 
+		  d2, nk1, d2min[nk1]);
 #endif
 	  d2min[nk1] = d2;
-	  dmin[nk1] = sqrt(d2);
 	  /* bubble sort */
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] before bubble sort:");
+	  Rprintf("\td2min[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 #endif
 	  unsorted = TRUE;
 	  for(k = nk1; unsorted && k > 0; k--) {
 	    k1 = k - 1;
-	    if(dmin[k] < dmin[k1]) {
+	    if(d2min[k] < d2min[k1]) {
 	      /* swap entries */
-	      tmp = dmin[k1];
-	      tmp2 = d2min[k1];
-	      dmin[k1] = dmin[k];
+	      tmp = d2min[k1];
 	      d2min[k1] = d2min[k];
-	      dmin[k] = tmp;
-	      d2min[k] = tmp2;
+	      d2min[k] = tmp;
 	    } else {
 	      unsorted = FALSE;
 	    }
 	  }
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] after bubble sort:");
+	  Rprintf("\td2min[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 #endif
 	  /* adjust maximum distance */
-	  dminK  = dmin[nk1];
 	  d2minK = d2min[nk1];
 	}
       }
 
-    /* search forward */
-    for(right = i + 1;
-	right < npoints && (dx0 = (x[right * mdimen] - xi0)) < dminK ;
-	++right)
-      {
+      /* search forward */
+      for(right = i + 1; right < npoints; ++right) {
 
 #ifdef SPATSTAT_DEBUG
 	Rprintf("R=%d\n", right);
 	Rprintf("\t 0 ");
 #endif
+	dx0 = x[right * mdimen] - xi0;
 	d2 = dx0 * dx0; 
-	if(mdimen > 1) {
-	  for(j = 1; j < mdimen && d2 < d2minK; j++) {
+	if(d2 > d2minK)
+	  break;
+	for(j = 1; j < mdimen && d2 < d2minK; j++) {
 #ifdef SPATSTAT_DEBUG
-	    Rprintf("%d ", j);
+	  Rprintf("%d ", j);
 #endif
-	    dxj = xi[j] - x[right * mdimen + j];
-	    d2 += dxj * dxj;
-	  }
+	  dxj = xi[j] - x[right * mdimen + j];
+	  d2 += dxj * dxj;
 	}
 #ifdef SPATSTAT_DEBUG
-	Rprintf("\n\t sqrt(d2)=%lf\n", sqrt(d2));
+	Rprintf("\n\t d2=%lf\n", d2);
 #endif
 	if (d2 < d2minK) {
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tsqrt(d2)=%lf overwrites dmin[%d] = %lf\n", 
-		  sqrt(d2), nk1, dmin[nk1]);
+	  Rprintf("\td2=%lf overwrites d2min[%d] = %lf\n", 
+		  d2, nk1, d2min[nk1]);
 #endif
 	  /* overwrite last entry */
 	  d2min[nk1] = d2;
-	  dmin[nk1] = sqrt(d2);
-#ifdef SPATSTAT_DEBUG
-	  Rprintf("sqrt(d2)=%lf\n", sqrt(d2));
-#endif
 	  /* bubble sort */
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] before bubble sort:");
+	  Rprintf("\td2min[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 #endif
 	  unsorted = TRUE;
 	  for(k = nk1; unsorted && k > 0; k--) {
 	    k1 = k - 1;
-	    if(dmin[k] < dmin[k1]) {
+	    if(d2min[k] < d2min[k1]) {
 	      /* swap entries */
-	      tmp = dmin[k1];
-	      tmp2 = d2min[k1];
-	      dmin[k1] = dmin[k];
+	      tmp = d2min[k1];
 	      d2min[k1] = d2min[k];
-	      dmin[k] = tmp;
-	      d2min[k] = tmp2;
+	      d2min[k] = tmp;
 	    } else {
 	      unsorted = FALSE;
 	    }
 	  }
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] after bubble sort:");
+	  Rprintf("\td2min[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 #endif
 	  /* adjust maximum distance */
-	  dminK  = dmin[nk1];
 	  d2minK = d2min[nk1];
 	}
       }
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n");
+      Rprintf("\n");
 #endif
 
-    /* copy nn distances for point i 
-       to output matrix in ROW MAJOR order
-    */
-    for(k = 0; k < nk; k++) {
-      nnd[nk * i + k] = dmin[k];
+      /* copy nn distances for point i 
+	 to output matrix in ROW MAJOR order
+      */
+      for(k = 0; k < nk; k++) {
+	nnd[nk * i + k] = sqrt(d2min[k]);
+      }
     }
   }
 }
@@ -670,7 +628,7 @@ void knndMD(n, m, kmax, x, nnd, huge)
 
 
 void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
-     /* inputs */
+/* inputs */
      int *n, *m, *kmax;
      double *x, *huge;
      /* output matrix (kmax * npoints) */
@@ -678,9 +636,10 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
      int *nnwhich;
 { 
   int npoints, mdimen, nk, nk1, i, j, k, k1, left, right, unsorted, itmp;
-  double d2, dminK, d2minK, xi0, dx0, dxj, hu, hu2, tmp, tmp2;
-  double *dmin, *d2min, *xi;
+  double d2, d2minK, xi0, dx0, dxj, hu, hu2, tmp;
+  double *d2min, *xi;
   int *which;
+  int maxchunk;
 
   hu = *huge;
   hu2 = hu * hu;
@@ -695,7 +654,6 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
      for the current point
   */
 
-  dmin = (double *) R_alloc((size_t) nk, sizeof(double));
   d2min = (double *) R_alloc((size_t) nk, sizeof(double));
   which = (int *) R_alloc((size_t) nk, sizeof(int));
 
@@ -703,74 +661,68 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
      scratch space
   */
   xi = (double *) R_alloc((size_t) mdimen, sizeof(double));
-  /*  dx = (double *) R_alloc((size_t) mdimen, sizeof(double)); */
 
   /* loop over points */
 
-  for(i = 0; i < npoints; i++) {
-
+  OUTERCHUNKLOOP(i, npoints, maxchunk, 16384) {
     R_CheckUserInterrupt();
+    INNERCHUNKLOOP(i, npoints, maxchunk, 16384) {
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\ni=%d\n", i); 
+      Rprintf("\ni=%d\n", i); 
 #endif
 
-    /* initialise nn distances */
+      /* initialise nn distances */
 
-    dminK  = hu;
-    d2minK = hu2;
-    for(k = 0; k < nk; k++) {
-      dmin[k] = hu;
-      d2min[k] = hu2;
-      which[k] = -1;
-    }
+      d2minK = hu2;
+      for(k = 0; k < nk; k++) {
+	d2min[k] = hu2;
+	which[k] = -1;
+      }
 
-    for(j = 0; j < mdimen; j++)
-      xi[j] = x[i* mdimen + j];
-    xi0 = xi[0];
+      for(j = 0; j < mdimen; j++)
+	xi[j] = x[i* mdimen + j];
+      xi0 = xi[0];
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n (");
-    for(j = 0; j < mdimen; j++)
-      Rprintf("%lf, ", x[i * mdimen + j]);
-    Rprintf(")\n");
+      Rprintf("\n (");
+      for(j = 0; j < mdimen; j++)
+	Rprintf("%lf, ", x[i * mdimen + j]);
+      Rprintf(")\n");
 #endif
 
-    /* search backward */
-    for(left = i - 1;
-        left >= 0 && (dx0 = (xi0 - x[left * mdimen])) < dminK ;
-	--left)
-      {
+      /* search backward */
+      for(left = i - 1; left >= 0; --left) {
 
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("L=%d, dminK=%lf\n", left, dminK);
-	  Rprintf("\t 0 ");
+	Rprintf("L=%d, d2minK=%lf\n", left, d2minK);
+	Rprintf("\t 0 ");
 #endif
-
+	dx0 = xi0 - x[left * mdimen];
 	d2 = dx0 * dx0; 
-	if(mdimen > 1) {
-	  for(j = 1; j < mdimen && d2 < d2minK; j++) {
+	if(d2 > d2minK)
+	  break;
+
+	for(j = 1; j < mdimen && d2 < d2minK; j++) {
 #ifdef SPATSTAT_DEBUG
-	    Rprintf("%d ", j);
+	  Rprintf("%d ", j);
 #endif
-	    dxj = xi[j] - x[left * mdimen + j];
-	    d2 += dxj * dxj;
-	  }
+	  dxj = xi[j] - x[left * mdimen + j];
+	  d2 += dxj * dxj;
 	}
 	if (d2 < d2minK) {
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tsqrt(d2)=%lf overwrites dmin[%d] = %lf\n", 
-		  sqrt(d2), nk1, dmin[nk1]);
+	  Rprintf("\td2=%lf overwrites d2min[%d] = %lf\n", 
+		  d2, nk1, d2min[nk1]);
 #endif
 	  /* overwrite last entry */
 	  d2min[nk1] = d2;
-	  dmin[nk1] = sqrt(d2);
 	  which[nk1] = left;
 	  /* bubble sort */
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] before bubble sort:");
+	  Rprintf("\td2min[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 	  Rprintf("\twhich[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
@@ -780,25 +732,22 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
 	  unsorted = TRUE;
 	  for(k = nk1; unsorted && k > 0; k--) {
 	    k1 = k - 1;
-	    if(dmin[k] < dmin[k1]) {
+	    if(d2min[k] < d2min[k1]) {
 	      /* swap entries */
-	      tmp = dmin[k1];
-	      tmp2 = d2min[k1];
-	      itmp = which[k1];
-	      dmin[k1] = dmin[k];
+	      tmp = d2min[k1];
 	      d2min[k1] = d2min[k];
+	      d2min[k] = tmp;
+	      itmp = which[k1];
 	      which[k1] = which[k];
-	      dmin[k] = tmp;
-	      d2min[k] = tmp2;
 	      which[k] = itmp;
 	    } else {
 	      unsorted = FALSE;
 	    }
 	  }
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] after bubble sort:");
+	  Rprintf("\td2min[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 	  Rprintf("\twhich[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
@@ -806,45 +755,41 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
 	  Rprintf("\n");
 #endif
 	  /* adjust maximum distance */
-	  dminK  = dmin[nk1];
 	  d2minK = d2min[nk1];
 	}
       }
 
-    /* search forward */
-    for(right = i + 1;
-	right < npoints && (dx0 = (x[right * mdimen] - xi0)) < dminK ;
-	++right)
-      {
+      /* search forward */
+      for(right = i + 1; right < npoints; ++right) {
 
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("R=%d, dminK=%lf\n", right, dminK);
-	  Rprintf("\t 0 ");
+	Rprintf("R=%d, d2minK=%lf\n", right, d2minK);
+	Rprintf("\t 0 ");
 #endif
+	dx0 = x[right * mdimen] - xi0;
 	d2 = dx0 * dx0; 
-	if(mdimen > 1) {
-	  for(j = 1; j < mdimen && d2 < d2minK; j++) {
+	if(d2 > d2minK) 
+	  break;
+	for(j = 1; j < mdimen && d2 < d2minK; j++) {
 #ifdef SPATSTAT_DEBUG
-	    Rprintf("%d ", j);
+	  Rprintf("%d ", j);
 #endif
-	    dxj = xi[j] - x[right * mdimen + j];
-	    d2 += dxj * dxj;
-	  }
+	  dxj = xi[j] - x[right * mdimen + j];
+	  d2 += dxj * dxj;
 	}
 	if (d2 < d2minK) {
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tsqrt(d2)=%lf overwrites dmin[%d] = %lf\n", 
-		  sqrt(d2), nk1, dmin[nk1]);
+	  Rprintf("\td2=%lf overwrites d2min[%d] = %lf\n", 
+		  d2, nk1, d2min[nk1]);
 #endif
 	  /* overwrite last entry */
 	  d2min[nk1] = d2;
-	  dmin[nk1] = sqrt(d2);
 	  which[nk1] = right;
 	  /* bubble sort */
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] before bubble sort:");
+	  Rprintf("\td2min[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 	  Rprintf("\twhich[] before bubble sort:");
 	  for(k = 0; k < nk; k++)
@@ -854,25 +799,22 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
 	  unsorted = TRUE;
 	  for(k = nk1; unsorted && k > 0; k--) {
 	    k1 = k - 1;
-	    if(dmin[k] < dmin[k1]) {
+	    if(d2min[k] < d2min[k1]) {
 	      /* swap entries */
-	      tmp = dmin[k1];
-	      tmp2 = d2min[k1];
-	      itmp = which[k1];
-	      dmin[k1] = dmin[k];
+	      tmp = d2min[k1];
 	      d2min[k1] = d2min[k];
+	      d2min[k] = tmp;
+	      itmp = which[k1];
 	      which[k1] = which[k];
-	      dmin[k] = tmp;
-	      d2min[k] = tmp2;
 	      which[k] = itmp;
 	    } else {
 	      unsorted = FALSE;
 	    }
 	  }
 #ifdef SPATSTAT_DEBUG
-	  Rprintf("\tdmin[] after bubble sort:");
+	  Rprintf("\td2min[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
-	    Rprintf("%lf, ", dmin[k]);
+	    Rprintf("%lf, ", d2min[k]);
 	  Rprintf("\n");
 	  Rprintf("\twhich[] after bubble sort:");
 	  for(k = 0; k < nk; k++)
@@ -880,23 +822,22 @@ void knnwMD(n, m, kmax, x, nnd, nnwhich, huge)
 	  Rprintf("\n");
 #endif
 	  /* adjust maximum distance */
-	  dminK  = dmin[nk1];
 	  d2minK = d2min[nk1];
 	}
       }
 
 #ifdef SPATSTAT_DEBUG
-    Rprintf("\n");
+      Rprintf("\n");
 #endif
 
-    /* copy nn distances for point i 
-       to output matrix in ROW MAJOR order
-    */
-    for(k = 0; k < nk; k++) {
-      nnd[nk * i + k] = dmin[k];
-      nnwhich[nk * i + k] = which[k];
+      /* copy nn distances for point i 
+	 to output matrix in ROW MAJOR order
+      */
+      for(k = 0; k < nk; k++) {
+	nnd[nk * i + k] = sqrt(d2min[k]);
+	/* convert index back to R convention */
+	nnwhich[nk * i + k] = which[k] + 1;
+      }
     }
   }
 }
-
-
